'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.forEachPath = forEachPath;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _reactRouter = require('react-router');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var RenderPagePlugin = function () {
  function RenderPagePlugin(_ref) {
    var inlineCSS = _ref.inlineCSS;
    (0, _classCallCheck3.default)(this, RenderPagePlugin);

    this.inlineCSS = inlineCSS;
    this.compiler = null;
  }

  (0, _createClass3.default)(RenderPagePlugin, [{
    key: 'apply',
    value: function apply(compiler) {
      var _this = this;

      this.compiler = compiler;
      this.compiler.plugin('emit', function (compilation, done) {
        var _evalBundle = (0, _utils.evalBundle)(compilation.assets);

        var route = _evalBundle.route;
        var Meta = _evalBundle.Meta;
        var Site = _evalBundle.Site;

        var css = compilation.assets['bundle.css'];

        cleanAssets(compilation.assets);

        function addToAssets(path, markup) {
          compilation.assets[routePathToAssetPath(path)] = createAssetFromContents(markup);
        }

        var tasks = [];
        forEachPath(route, function (path) {
          _this.compiler.debug('rendering path', path);
          tasks.push(_this.renderPath(route, path, css, Site, Meta).then(function (markup) {
            if (markup) {
              addToAssets(path, markup);
            }
          }, function (error) {
            _this.compiler.debug('error while rendering path', path);
            throw error;
          }));
        });

        _bluebird2.default.all(tasks).then(function () {
          done();
        }, function (err) {
          _this.compiler.debug('error', err);
          compilation.errors.push(err);
          done(err);
        });
      });
    }
  }, {
    key: 'renderPath',
    value: function renderPath(route, path, css, Site, Meta) {
      var _this2 = this;

      return new _bluebird2.default(function (resolve, reject) {
        var location = joinPath(_this2.compiler.options.output.publicPath, path);
        (0, _reactRouter.match)({ routes: route, location: location }, function (error, redirectLocation, routeProps) {
          if (error) {
            reject(error);
          } else if (!routeProps || redirectLocation) {
            resolve(null);
          } else {
            var innerMarkup = (0, _server.renderToString)(_react2.default.createElement(_reactRouter.RouterContext, routeProps));
            var meta = Meta.rewind();
            var markup = (0, _server.renderToStaticMarkup)(_react2.default.createElement(Site, {
              meta: meta,
              bundle: {
                js: _this2.compiler.options.output.publicPath + 'bundle.js',
                css: !_this2.inlineCSS && _this2.compiler.options.output.publicPath + 'bundle.css'
              },
              style: _this2.inlineCSS && (0, _utils.assetSource)(css),
              content: innerMarkup
            }));
            resolve(markup);
          }
        });
      });
    }
  }]);
  return RenderPagePlugin;
}(); /**
      * @copyright 2016-present, Sitegen team
      */

exports.default = RenderPagePlugin;


function createAssetFromContents(contents) {
  return {
    source: function source() {
      return contents;
    },
    size: function size() {
      return contents.length;
    }
  };
}

function routePathToAssetPath(path) {
  path = path.replace(/^\//, '').replace(/\/$/, '');
  return path + '/index.html';
}

function cleanAssets(assets) {
  (0, _keys2.default)(assets).forEach(function (key) {
    return delete assets[key];
  });
}

function normalizePath(href) {
  return href.replace(new RegExp('/+', 'g'), '/');
}

function joinPath() {
  for (var _len = arguments.length, segments = Array(_len), _key = 0; _key < _len; _key++) {
    segments[_key] = arguments[_key];
  }

  return normalizePath(segments.join('/'));
}

function forEachPath(route, func) {
  var trace = arguments.length <= 2 || arguments[2] === undefined ? [''] : arguments[2];

  var path = '/' + trace.filter(Boolean).join('/');
  if (route.params) {
    (0, _keys2.default)(route.params).forEach(function (key) {
      route.params[key].forEach(function (value) {
        func(interpolateParams(path, (0, _defineProperty3.default)({}, key, value)));
      });
    });
  } else {
    func(path);
  }
  if (route.childRoutes) {
    route.childRoutes.forEach(function (route) {
      return forEachPath(route, func, trace.concat(route.path));
    });
  }
}

function interpolateParams(path, params) {
  return path.replace(/:([a-zA-Z_\-]+)/g, function (_, key) {
    return params[key];
  });
}
