'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.createCompiler = createCompiler;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _config = require('../config');

var _RenderPagePlugin = require('./RenderPagePlugin');

var _RenderPagePlugin2 = _interopRequireDefault(_RenderPagePlugin);

var _LogProgressPlugin = require('./LogProgressPlugin');

var _LogProgressPlugin2 = _interopRequireDefault(_LogProgressPlugin);

var _PromiseAssetsPlugin = require('./PromiseAssetsPlugin');

var _PromiseAssetsPlugin2 = _interopRequireDefault(_PromiseAssetsPlugin);

var _MetaRegistryPlugin = require('./MetaRegistryPlugin');

var _MetaRegistryPlugin2 = _interopRequireDefault(_MetaRegistryPlugin);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright 2016-present, Sitegen team
 */

var BOOT_LOADER = require.resolve('../loader/boot');
var META_LOADER = require.resolve('../loader/meta');

var REACT_HOT_LOADER_PATCH = require.resolve('react-hot-loader/patch');
var REACT_HOT_LOADER_PLUGIN = require.resolve('react-hot-loader/babel');

var WEBPACK_HOT_CLIENT = require.resolve('webpack-hot-middleware/client');

function createCompiler(_ref) {
  var entry = _ref.entry;
  var output = _ref.output;
  var publicPath = _ref.publicPath;
  var env = _ref.env;
  var inlineCSS = _ref.inlineCSS;
  var progress = _ref.progress;

  var __DEBUG__ = env === 'production' ? undefined : (0, _stringify2.default)(process.env.DEBUG);

  var ctx = {
    basedir: _path2.default.dirname(entry),
    env: {
      development: env === 'development',
      production: env === 'production',
      content: env === 'content'
    }
  };

  var site = (0, _config.readConfigSync)(entry);

  var sitegenConfig = {

    site: site,

    context: ctx.basedir,

    entry: [ctx.env.development && REACT_HOT_LOADER_PATCH, ctx.env.development && WEBPACK_HOT_CLIENT, (0, _config.moduleRequest)(entry, BOOT_LOADER)],

    env: env,

    target: ctx.env.content ? 'node' : 'web',

    bail: ctx.env.content,

    output: {
      path: output || '/build',
      filename: 'bundle.js',
      chunkFilename: 'bundle[name].js',
      publicPath: publicPath
    },

    babel: {
      plugins: [ctx.env.development && REACT_HOT_LOADER_PLUGIN]
    },

    resolve: {
      alias: {
        'debug': require.resolve('debug/browser')
      }
    },

    plugins: [progress && new _webpack2.default.ProgressPlugin(progress), new _LogProgressPlugin2.default(env), new _webpack2.default.DefinePlugin({
      '__DEBUG__': __DEBUG__,
      'process.env.NODE_ENV': (0, _stringify2.default)(env)
    }), ctx.env.development && new _webpack2.default.optimize.OccurenceOrderPlugin(), ctx.env.development && new _webpack2.default.HotModuleReplacementPlugin(), ctx.env.development && new _webpack2.default.NoErrorsPlugin(), ctx.env.development && new _PromiseAssetsPlugin2.default({
      name: 'promiseBundle',
      then: _utils.evalBundle
    }), ctx.env.content && new _RenderPagePlugin2.default({
      inlineCSS: inlineCSS
    }), new _MetaRegistryPlugin2.default({ loader: META_LOADER })]
  };

  var configPipeline = [];
  configPipeline = configPipeline.concat((0, _config.defaultConfig)(ctx));
  configPipeline = configPipeline.concat(site.plugins.map(function (p) {
    return p.configure(ctx);
  }));
  configPipeline = configPipeline.concat(site.configure(ctx), sitegenConfig);

  var config = (0, _config.configureWebpack)(_config.mergeConfig.apply(undefined, (0, _toConsumableArray3.default)(configPipeline)));

  return (0, _webpack2.default)(config);
}
