'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.injectStyleLoader = exports.CSS = exports.img = exports.url = exports.markdown = exports.JS = exports.extractCSSPlugin = undefined;

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.defaultConfig = defaultConfig;
exports.mergeConfig = mergeConfig;
exports.moduleRequest = moduleRequest;
exports.loader = loader;
exports.configureWebpack = configureWebpack;
exports.readConfigSync = readConfigSync;
exports.injectCSS = injectCSS;
exports.extractCSS = extractCSS;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _resolve = require('resolve');

var _babelCore = require('babel-core');

var _lodash = require('lodash');

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _minimatch = require('minimatch');

var _utils = require('./compiler/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NODE_MODULES_RE = /node_modules/; /**
                                       * @copyright 2016-present, Sitegen team
                                       * 
                                       */

var BABEL_PRESET_ES2015 = require.resolve('babel-preset-es2015');
var BABEL_PRESET_STAGE_1 = require.resolve('babel-preset-stage-1');
var BABEL_PRESET_REACT = require.resolve('babel-preset-react');

var SITEGEN_COMPONENTS = require.resolve('./components');

function defaultConfig(_ref) {
  var env = _ref.env;


  var deployCSS = env.development ? injectCSS : extractCSS;

  return {

    devtool: env.development ? 'cheap-module-eval-source-map' : undefined,

    babel: {
      presets: [BABEL_PRESET_ES2015, BABEL_PRESET_STAGE_1, BABEL_PRESET_REACT]
    },

    loaders: {
      '**/*.js': JS,
      '**/*.md': [JS, markdown({ defaultComponents: SITEGEN_COMPONENTS })]
    },

    globalLoaders: {
      '**/*.css': deployCSS(CSS({ minimize: env.production })),
      '**/*.png': img,
      '**/*.jpg': img,
      '**/*.jpeg': img,
      '**/*.gif': img,
      '**/*.ico': img
    },

    plugins: [(env.content || env.production) && extractCSSPlugin, env.production && new _webpack2.default.optimize.UglifyJsPlugin({
      compress: { warnings: false }
    })]
  };
}

function mergeConfig() {
  for (var _len = arguments.length, config = Array(_len), _key = 0; _key < _len; _key++) {
    config[_key] = arguments[_key];
  }

  config = config.filter(Boolean);
  (0, _invariant2.default)(config.length > 0, 'Trying to merge empty list of configurations');
  if (config.length === 1) {
    return config[0];
  } else {
    return config.reduce(mergeConfigImpl);
  }
}

function mergeConfigImpl(a, b) {

  return (0, _extends3.default)({}, a, b, {
    entry: mergeArray(a.entry, b.entry),
    babel: (0, _extends3.default)({}, a.babel, b.babel, {
      presets: mergeArray(a.babel && a.babel.presets, b.babel && b.babel.presets),
      plugins: mergeArray(a.babel && a.babel.plugins, b.babel && b.babel.plugins)
    }),
    output: (0, _extends3.default)({}, a.output, b.output),
    loaders: (0, _extends3.default)({}, a.loaders, b.loaders),
    globalLoaders: (0, _extends3.default)({}, a.globalLoaders, b.globalLoaders),
    plugins: mergeArray(a.plugins, b.plugins)
  });
}

function mergeArray() {
  var a = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
  var b = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  var _a = a || [];
  var _b = b || [];
  return [].concat(_a).concat(_b).filter(function (item) {
    return item;
  });
}

function moduleRequest(id) {
  for (var _len2 = arguments.length, loaders = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    loaders[_key2 - 1] = arguments[_key2];
  }

  loaders.reverse();
  var loader = (0, _lodash.flatten)(loaders);
  return normalizeLoader(loader) + '!' + id;
}

function loader(loader) {
  return loaderImpl({ loader: loader });
}

function loaderImpl(config) {
  var mergeQuery = config.mergeQuery || function (a, b) {
    return (0, _extends3.default)({}, a, b);
  };
  var configurator = function configurator(query) {
    return loaderImpl((0, _extends3.default)({}, config, {
      query: mergeQuery(config.query, query)
    }));
  };
  configurator.config = config;
  configurator.toLoaderString = function toLoaderString() {
    return normalizeLoader(config);
  };
  configurator.toString = configurator.toLoaderString;
  return configurator;
}

function normalizeLoader(element) {
  if (Array.isArray(element)) {
    return element.map(normalizeLoader).join('!');
  } else if (typeof element === 'string') {
    return element;
  } else if (element.toLoaderString) {
    return element.toLoaderString();
  } else {
    if (element.query) {
      return element.loader + '?' + (0, _stringify2.default)(element.query);
    } else {
      return element.loader;
    }
  }
}

function makePatterMatcher(context, pattern, global) {
  if (!global) {
    pattern = _path2.default.join(context, pattern);
  }
  var patterMatcher = new _minimatch.Minimatch(pattern, { dot: true });

  var matcher = function matcher(filename) {
    return global ? patterMatcher.match(filename) : !NODE_MODULES_RE.exec(filename) && patterMatcher.match(filename);
  };

  matcher.toString = function () {
    return '[PatternMatcher ' + pattern + ']';
  };
  matcher.inspect = matcher.toString;

  return matcher;
}

function configureWebpackLoaderList(context, loaders, global) {
  var result = [];
  for (var _pattern in loaders) {
    var _loader = loaders[_pattern];
    result.push({
      loader: normalizeLoader(_loader),
      test: makePatterMatcher(context, _pattern, global)
    });
  }
  return result;
}

function configureWebpack(_ref2) {
  var context = _ref2.context;
  var loaders = _ref2.loaders;
  var globalLoaders = _ref2.globalLoaders;
  var config = (0, _objectWithoutProperties3.default)(_ref2, ['context', 'loaders', 'globalLoaders']);

  var webpackLoaders = configureWebpackLoaderList(context, loaders);
  var webpackGlobalLoaders = configureWebpackLoaderList(context, globalLoaders, true);
  return (0, _extends3.default)({}, config, {
    context: context,
    output: (0, _extends3.default)({}, config.output, {
      path: _path2.default.resolve(config.context, config.output.path)
    }),
    module: {
      loaders: webpackLoaders.concat(webpackGlobalLoaders)
    }
  });
}

function noopConfigure(_ctx) {
  return {};
}

function readConfigSync(filename) {
  var basedir = _path2.default.dirname(filename);
  // $FlowIssue: ...

  var _requireFile = requireFile(filename);

  var _requireFile$plugins = _requireFile.plugins;
  var plugins = _requireFile$plugins === undefined ? [] : _requireFile$plugins;
  var _requireFile$configur = _requireFile.configure;
  var configure = _requireFile$configur === undefined ? noopConfigure : _requireFile$configur;
  var route = _requireFile.route;

  return {
    route: route,
    configure: configure,
    // $FlowIssue: ...
    plugins: plugins.map(function (p) {
      return requireModule(p, { basedir: basedir });
    })
  };
}

function requireModule(module, _ref3) {
  var basedir = _ref3.basedir;

  var filename = (0, _resolve.sync)(module, { basedir: basedir });
  return requireFile(filename);
}

function requireFile(filename) {
  var source = _fs2.default.readFileSync(filename, 'utf8');
  source = (0, _babelCore.transform)(source, {
    filename: filename,
    presets: [BABEL_PRESET_ES2015, BABEL_PRESET_STAGE_1]
  }).code;
  return (0, _utils.evalAsModule)(source, filename);
}

var extractCSSPlugin = exports.extractCSSPlugin = new _extractTextWebpackPlugin2.default('bundle.css');

var JS = exports.JS = loader(require.resolve('babel-loader'));

var markdown = exports.markdown = loader(require.resolve('reactdown/webpack'));

var url = exports.url = loader(require.resolve('url-loader'));

var img = exports.img = url({ limit: 10000 });

var CSS = exports.CSS = loader(require.resolve('css-loader'));

var injectStyleLoader = exports.injectStyleLoader = loader(require.resolve('style-loader'));

function injectCSS() {
  for (var _len3 = arguments.length, loaders = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    loaders[_key3] = arguments[_key3];
  }

  var loader = (0, _lodash.flatten)(loaders);
  // $FlowIssue: TODO: Investigate
  return normalizeLoader([injectStyleLoader].concat(loader)).split('!');
}

function extractCSS() {
  for (var _len4 = arguments.length, loaders = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    loaders[_key4] = arguments[_key4];
  }

  var loader = (0, _lodash.flatten)(loaders);
  return extractCSSPlugin.extract(normalizeLoader(injectStyleLoader), normalizeLoader(loader)).split('!');
}
