'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AssetRoute = exports.CollectionRoute = exports.PageRoute = exports.Route = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

exports.page = page;
exports.collection = collection;
exports.asset = asset;
exports.forEach = forEach;

var _minimatch = require('minimatch');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Marker class for site routes.
 */

var Route = exports.Route = function Route() {
  (0, _classCallCheck3.default)(this, Route);
};

/**
 * Mapping from path segments to routes.
 */
/**
 * @copyright 2016-present, Sitegen team
 * 
 */

/**
 * Route which represents a regular page.
 */

var PageRoute = exports.PageRoute = function (_Route) {
  (0, _inherits3.default)(PageRoute, _Route);

  function PageRoute(component, options, routes) {
    (0, _classCallCheck3.default)(this, PageRoute);

    var _this = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(PageRoute).call(this));

    _this.component = component;
    _this.options = options;
    _this.routes = routes;
    return _this;
  }

  return PageRoute;
}(Route);

/**
 * Route which represents a page which manages a collection of pages.
 *
 * Example blog post index which manages a collection of blog posts.
 */

var CollectionRoute = exports.CollectionRoute = function (_Route2) {
  (0, _inherits3.default)(CollectionRoute, _Route2);

  function CollectionRoute(component, collection, options) {
    (0, _classCallCheck3.default)(this, CollectionRoute);

    var _this2 = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(CollectionRoute).call(this));

    _this2.component = component;
    _this2.collection = collection;
    _this2.options = options;
    return _this2;
  }

  return CollectionRoute;
}(Route);

/**
 * Route which references some asset.
 */


var AssetRoute = exports.AssetRoute = function (_Route3) {
  (0, _inherits3.default)(AssetRoute, _Route3);

  function AssetRoute(req) {
    (0, _classCallCheck3.default)(this, AssetRoute);

    var _this3 = (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(AssetRoute).call(this));

    _this3.req = req;
    return _this3;
  }

  return AssetRoute;
}(Route);

function page(component) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var routes = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];


  return new PageRoute(component, options, routes);
}

function collection(component, pattern) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var spec = parsePattern(pattern);
  return new CollectionRoute(component, spec, options);
}

function asset(req) {
  return new AssetRoute(req);
}

function forEach(route, func) {
  func(route);
  if (route instanceof PageRoute) {
    for (var k in route.routes) {
      forEach(route.routes[k], func);
    }
  }
}

function parsePattern(pattern) {
  var matcher = new _minimatch.Minimatch(pattern);
  var context = [];
  for (var i = 0; i < matcher.set[0].length; i++) {
    if (typeof matcher.set[0][i] === 'string') {
      context.push(matcher.set[0][i]);
    } else {
      break;
    }
  }
  if (context.length > 1) {
    context = context.join('/');
  } else {
    context = '.';
  }
  return {
    context: context,
    match: matcher.match.bind(matcher)
  };
}
